// Copyright (C) 1996 T.Kamachi

import java.io.*;
import java.util.*;

public class MailMessage {
  private DataInputStream si;
  private String currentLine;
  private String peekString;
  private String name;
  private String body;
  private Hashtable fields = new Hashtable();

  MailMessage(File file) throws IOException {
    this(new FileInputStream(file));
  }

  MailMessage(InputStream s) throws IOException {
    si = new DataInputStream(s);
    parse();
    si.close();
  }

  String getField(String name) {
    return (String)fields.get(name.toUpperCase());
  }

  public void parse() {
    try {
      while (true) {
        readField();
        if (name==null) {
          break;
        }
        fields.put(name.toUpperCase(), body);
      }
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  private void readField() throws IOException {
    name = null;
    body = null;
    String l = getNextLine();
    parseLine(l);
    if (name==null) {
      return;
    }
    while (true) {
      l = getNextLine();
      if (l!=null && l.length()!=0 && Character.isSpace(l.charAt(0))) {
        char c1[] = new char[1];
        c1[0] = '\n';
        body = body.concat(new String(c1)+l);
      } else {
        pushBack();
        break;
      }
    }
  }

  private void parseLine(String l) {
    if (l==null) {
      return;
    }
    int i = l.indexOf(":");
    if (i<0) {
      return;
    }
    name = l.substring(0, i);
    body = l.substring(i+1);
    return;
  }

  private String getNextLine() throws IOException {
    if (peekString!=null) {
      String str = peekString;
      peekString = null;
      return str;
    } else {
      currentLine = si.readLine();
      return currentLine;
    }
  }

  private void pushBack() {
    peekString = currentLine;
  }
}
