// Copyright (C) 1996 T.Kamachi

import java.io.*;
import java.util.*;

class MailSyncHost {
  public static void main(String arg[]) throws IOException {
    if (arg.length!=2) {
      System.out.println("usage: MailSyncHost Mail-dir sync-file");
      return;
    }

    File sf = new File(arg[1]);
    SyncFile sync = new SyncFile(sf);

    File dir = new File(arg[0]);
    String files[] = dir.list();
    if (files==null) {
      System.err.println("No folders");
      return;
    }
    HostMailFolder folder;
    for (int i=0;i<files.length;i++) {
      File f = new File(dir.toString()+ File.separatorChar + files[i]);
      if (f.isDirectory()) {
        if (!f.getName().equals("inbox")) {
          continue;
        }
        folder = new HostMailFolder(f, sync);
        folder.process();
      }
    }
  }
}

class SyncFile {
  Hashtable folders = new Hashtable();

  public SyncFile(File file) throws IOException {
    FileInputStream fi = new FileInputStream(file);
    DataInputStream di = new DataInputStream(fi);
    read(di);
  }
  public SyncFile(DataInputStream s) throws IOException {
    read(s);
  }
  public void read(DataInputStream s) throws IOException {
    try {
      String l, folder, id;
      int i;
      while (true) {
        l = s.readLine();
        if (l==null) {
          break;
        }
        i = l.indexOf(": ");
        folder = l.substring(0,i);
        id = l.substring(i+2);
        register(folder, id);
      }
    } catch (EOFException e) {
    }
  }

  public String getTargetFolder(String id) {
    Enumeration ef = folders.keys();
    String folder;
    Hashtable ftbl;
    while (ef.hasMoreElements()) {
      folder = (String)ef.nextElement();
      ftbl = (Hashtable)folders.get(folder);
      if (ftbl.containsKey(id.trim())) {
        ftbl.remove(id.trim());
        return folder;
      }
    }
    return null;
  }
  private Hashtable getFolderTable(String folder) {
    Hashtable ftbl = (Hashtable)folders.get(folder);
    if (ftbl==null) {
      ftbl = new Hashtable();
      folders.put(folder, ftbl);
    }
    return ftbl;
  }
  private void register(String folder, String id) {
    Hashtable ftbl = getFolderTable(folder);
    ftbl.put(id.trim(), new Object());
  }
}

class HostMailFolder extends MailFolder {
  private SyncFile sync;

  public HostMailFolder(File dir, SyncFile sf) throws IOException {
     super(dir);
     sync = sf;
  }

  public void output(String id, String mesNo) {
    String target = sync.getTargetFolder(id);
    if (target!=null) {
      if (target.equals("TRASH")) {
        System.out.println("rmm +inbox " + mesNo);
      } else {
        System.out.println("refile " + mesNo + " +" + target);
      }
    }
  }
}
